# -*- coding: utf-8 -*-
""" Mock classes used for testing which in turn really come from 
trepan.processor.command.mock """

from trepan.lib import default
from trepan.processor.command import mock

dbg_setup = mock.dbg_setup
